package sun.security.krb5.internal;

import sun.misc.IOUtils;
import java.io.IOException;
import java.net.InetSocketAddress;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.net.Socket;

class TCPClient extends NetClient
{
    private Socket tcpSocket;
    private BufferedOutputStream out;
    private BufferedInputStream in;
    
    TCPClient(String hostname, int port, int n) throws IOException {
        (this.tcpSocket = new Socket()).connect(new InetSocketAddress(hostname, port), n);
        this.out = new BufferedOutputStream(this.tcpSocket.getOutputStream());
        this.in = new BufferedInputStream(this.tcpSocket.getInputStream());
        this.tcpSocket.setSoTimeout(n);
    }
    
    @Override
    public void send(byte[] b) throws IOException {
        byte[] b2 = new byte[4];
        intToNetworkByteOrder(b.length, b2, 0, 4);
        this.out.write(b2);
        this.out.write(b);
        this.out.flush();
    }
    
    @Override
    public byte[] receive() throws IOException {
        byte[] array = new byte[4];
        int fully = this.readFully(array, 4);
        if (fully != 4) {
            if (Krb5.DEBUG) {
                System.out.println(">>>DEBUG: TCPClient could not read length field");
            }
            return null;
        }
        else {
            int networkByteOrderToInt = networkByteOrderToInt(array, 0, 4);
            if (Krb5.DEBUG) {
                System.out.println(">>>DEBUG: TCPClient reading " + networkByteOrderToInt + " bytes");
            }
            if (networkByteOrderToInt <= 0) {
                if (Krb5.DEBUG) {
                    System.out.println(">>>DEBUG: TCPClient zero or negative length field: " + networkByteOrderToInt);
                }
                return null;
            }
            else {
                try {
                    return IOUtils.readExactlyNBytes(this.in, networkByteOrderToInt);
                }
                catch (IOException ex) {
                    if (Krb5.DEBUG) {
                        System.out.println(">>>DEBUG: TCPClient could not read complete packet (" + networkByteOrderToInt + "/" + fully + ")");
                    }
                    return null;
                }
            }
        }
    }
    
    @Override
    public void close() throws IOException {
        this.tcpSocket.close();
    }
    
    private int readFully(byte[] b, int i) throws IOException {
        int off = 0;
        while (i > 0) {
            int read = this.in.read(b, off, i);
            if (read == -1) {
                return (off == 0) ? -1 : off;
            }
            off += read;
            i -= read;
        }
        return off;
    }
    
    private static int networkByteOrderToInt(byte[] array, int n, int n2) {
        if (n2 > 4) {
            throw new IllegalArgumentException("Cannot handle more than 4 bytes");
        }
        int n3 = 0;
        for (int i = 0; i < n2; ++i) {
            n3 = (n3 << 8 | (array[n + i] & 0xFF));
        }
        return n3;
    }
    
    private static void intToNetworkByteOrder(int n, byte[] array, int n2, int n3) {
        if (n3 > 4) {
            throw new IllegalArgumentException("Cannot handle more than 4 bytes");
        }
        for (int i = n3 - 1; i >= 0; --i) {
            array[n2 + i] = (byte)(n & 0xFF);
            n >>>= 8;
        }
        return;
    }
}
